import { InitAction } from "../action/InitAction";
import { Context } from "../context/Context";
import { ERuleTypes } from "../rules/enum/ERuleTypes";
import { InitRules } from "../rules/InitRules";
import { IManager } from "./interface/IManager";
import { v4 as uuidv4 } from "uuid";
import { EActionStatus } from "../action/enum/EActionStatus";
import { CreateClientOperation } from "../operation/CreateClientOperation";
import { CreateListenersOperation } from "../operation/CreateListenersOperation";
import { EMessageType } from "../logger/enum/EMessageType";
import { EOperationStatus } from "../operation/enum/EOperationStatus";
import { InitActionParams } from "../action/type/InitActionParams";
import { BaseManager } from "./base/BaseManager";
import { LeaveChannelOperation } from "../operation/LeaveChannelOperation";
import { DisconnectParams } from "../action/type/DisconnectParams";
import { DisconnectRules } from "../rules/DisconnectRules";
import { DisableTracksOperation } from "../operation/DisableTracksOperation";
import { CustomError } from "../exception/CustomError";
import { DisconnectAction } from "../action/DisconnectAction";
const parseError = require('parse-error');

export class DisconnectManager extends BaseManager implements IManager {

    constructor(action : DisconnectAction, context: Context){
        super(action, context);
    }

    /*@overwrite*/
    public async performAction(): Promise<EActionStatus> {

        return new Promise( async (resolve, reject)=>{

            let params : DisconnectParams = Object.assign(this.getAction().getParams()); 

            let validationRules = new DisconnectRules(params); 

            validationRules.validate([ERuleTypes.IS_INITIATED, ERuleTypes.IS_CLIENT_INIT], this.context);
        
            if(validationRules.passes){

                try {
                    let leaveChannelOperation  = new LeaveChannelOperation(this.context);
                        await leaveChannelOperation.handler(params);

                    let disableTracksOperation  = new DisableTracksOperation(this.context);
                        await disableTracksOperation.handler(params);


                        if(leaveChannelOperation.status == EOperationStatus.PERFORMED &&
                            disableTracksOperation.status == EOperationStatus.PERFORMED){

                            this.action.status = EActionStatus.PERFORMED;
            
                            this.logger.log({
                                id: uuidv4(),
                                title: this.action.type,
                                body: "Action "+this.action.type + " performaced",
                                action: this.action,
                                errors: validationRules.errors,
                                type:EMessageType.INFO
                            });

                        }else{

                            let errors : Array<CustomError> = [];
                            if(leaveChannelOperation.errors) errors = [...errors, ...leaveChannelOperation.errors];

                            this.action.status = EActionStatus.FAILED;
                            this.logger.log({
                                id: uuidv4(),
                                title: this.action.type,
                                body: "Something operations not performaced in action "+this.action.type,
                                action: this.action,
                                errors: errors,
                                type: EMessageType.ERROR
                            });

                        }

                } catch (error) {

                    let errors = parseError(error);

                    this.action.status = EActionStatus.FAILED;
                    
                    this.logger.log({
                        id: uuidv4(),
                        title: this.action.type,
                        body: "Something operations not performaced in action "+this.action.type,
                        action: this.action,
                        errors: errors,
                        type: EMessageType.ERROR
                    });
                }
                
            }else{
                
                this.action.status = EActionStatus.FAILED;
                      
                this.logger.log({
                    id: uuidv4(),
                    title: this.action.type,
                    body: "Action "+this.action.type + " not performaced",
                    action: this.action,
                    errors: validationRules.errors,
                    type: EMessageType.ERROR
                });
            }

            resolve(this.action.status);

        });

    }

}